#include "historyfolder.h"
#include <QTimer>
namespace Fm {

std::weak_ptr<HistoryFolderList> HistoryFolderList::globalInstance_;
bool HistoryFolderList::runing = false;

static inline CStrPtr get_legacy_history_file(void) {
    return CStrPtr{g_build_filename(g_get_home_dir(), ".history", nullptr)};
}

static inline CStrPtr get_new_history_file(void) {
    return CStrPtr{g_build_filename(g_get_user_config_dir(), "gtk-3.0", "history", nullptr)};
}

HistoryFolderList::HistoryFolderList(QObject* parent):
    QObject(parent),
    idle_handler{false} {
    auto fpath = get_new_history_file();
    file = FilePath::fromLocalPath(fpath.get());
    if (!file.isExists()) {
        auto fpathbak = get_legacy_history_file();
        file = FilePath::fromLocalPath(fpathbak.get());
    }
    load();

    mon = GObjectPtr<GFileMonitor>{g_file_monitor_file(file.gfile().get(), G_FILE_MONITOR_NONE, nullptr, nullptr), false};
    if(mon) {
        g_signal_connect(mon.get(), "changed", G_CALLBACK(_onFileChanged), this);
    }
}

void HistoryFolderList::onFileChanged(GFileMonitor* /*mon*/, GFile* /*gf*/, GFile* /*other*/, GFileMonitorEvent /*evt*/) {
    // reload the bookmarks
    hlist_.clear();
    load();
}

bool HistoryFolderList::judgeruning(){
    if(!runing){
        runing = true;
        return false;
    }
    return true;
}

std::shared_ptr<HistoryFolderList> HistoryFolderList::globalInstance() {
    auto hislist = globalInstance_.lock();
    if(!hislist) {
        hislist = std::make_shared<HistoryFolderList>();
        globalInstance_ = hislist;
    }
    return hislist;
}

void HistoryFolderList::load() {
    auto fpath = file.localPath();
    FILE* f;
    char buf[1024];
    /* load the file */
    f = fopen(fpath.get(), "r");
    if(f) {
        while(fgets(buf, 1024, f)) {
            // format of each line in the bookmark file:
            // <URI> <name>\n
            char* sep;
            sep = strchr(buf, '\n');
            if(sep) {
                *sep = '\0';
            }
            auto uri = buf;
            if(uri[0] != '\0') {
                hlist_.push_back(uri);
            }
        }
        fclose(f);
    }
}


void HistoryFolderList::save() {
    std::string buf;
    for(auto& item: hlist_) {
        buf += item.toUtf8().constData();
        buf += '\n';
    }
    idle_handler = false;
    GError* err = nullptr;
    if(!g_file_replace_contents(file.gfile().get(), buf.c_str(), buf.length(), nullptr,
                                FALSE, G_FILE_CREATE_NONE, nullptr, nullptr, &err)) {
        g_critical("%s", err->message);
        g_error_free(err);
    }
}

void HistoryFolderList::remove(const FilePath& path){
    for (auto item = hlist_.begin(); item != hlist_.end(); item++) {
        if((*item).compare(path.uri().get()) == 0){
            hlist_.erase(item);
            queueSave();
            return ;
        }
    }
}

void HistoryFolderList::insert(const FilePath& path) {
    if (path.isValid()){
        hlist_.push_back(path.uri().get());
        queueSave();
    }
}

void HistoryFolderList::queueSave() {
    save();
//    if(!idle_handler) {
//        QTimer::singleShot(0, this, &HistoryFolderList::save);
//        idle_handler = true;
//    }
}

void HistoryFolderList::clear() {
    hlist_.clear();
    queueSave();
}

bool HistoryFolderList::isempty() {
    return hlist_.empty();
}

}


