/*
 * Copyright (C) 2012 - 2015  Hong Jen Yee (PCMan) <pcman.tw@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


#ifndef FM_FOLDERITEMDELEGATE_H
#define FM_FOLDERITEMDELEGATE_H

#include "libfmqtglobals.h"
#include <QStyledItemDelegate>
#include "core/legacy/fm-config.h"
#include "folderview.h"
#include "utilities.h"
#include <QMap>
class QAbstractItemView;

namespace Fm {
class FolderView;
class FolderModelItemImple:public QObject{
    Q_OBJECT
public:
    FolderModelItemImple(std::shared_ptr<const Fm::FileInfo> info) : QObject(nullptr) {
        folder_ = Fm::Folder::fromPath(Fm::FilePath::fromPathStr(info->target().c_str()));
        connect(folder_.get(), &Fm::Folder::startLoading, this, &FolderModelItemImple::onFolderStartLoading);
        connect(folder_.get(), &Fm::Folder::finishLoading, this, &FolderModelItemImple::onFolderFinishLoading);
        connect(folder_.get(), &Fm::Folder::fileSystemChanged, this, &FolderModelItemImple::onFolderFsInfo);

    }
    bool findFile(QString target){
        if (target.compare(folder_->path().toString().get())){
            return true;
        }
        return false;
    }
    ~FolderModelItemImple(){
        qInfo()<<" free size";
    }
    void reload() {}
    QString getTotalSize() {
        return totsize_;
    }
    QString getFreeSize(){
        return freesize_;
    }
    int getPercent(){
        return free * 100 / total;
    }
private:
    void onFolderStartLoading(){}
    void onFolderFinishLoading(){}
    void onFolderFsInfo() {

        if(folder_->getFilesystemInfo(&total, &free)) {
            freesize_ = formatFileSize(free, fm_config->si_unit);
            freesize_.replace("GiB","G");
            totsize_ = formatFileSize(total, fm_config->si_unit);
            totsize_.replace("GiB","G");
        }
    }

private:
    std::shared_ptr<Folder> folder_;
    QString totsize_ = "";
    QString freesize_ = "";
    guint64 free, total;
};


class LIBFM_QT_API FolderItemDelegate : public QStyledItemDelegate {
    Q_OBJECT
public:
    explicit FolderItemDelegate(QAbstractItemView* view, QObject* parent = nullptr);

    virtual ~FolderItemDelegate();

    inline void setItemSize(QSize size) {
        itemSize_ = size;
    }

    inline QSize itemSize() const {
        return itemSize_;
    }

    inline void setIconSize(QSize size) {
        iconSize_ = size;
    }

    inline QSize iconSize() const {
        return iconSize_;
    }

    int fileInfoRole() {
        return fileInfoRole_;
    }

    void setFileInfoRole(int role) {
        fileInfoRole_ = role;
    }

    int iconInfoRole() {
        return iconInfoRole_;
    }

    void setIconInfoRole(int role) {
        iconInfoRole_ = role;
    }

    // only support vertical layout (icon view mode: text below icon)
    void setShadowColor(const QColor& shadowColor) {
      shadowColor_ = shadowColor;
    }

    // only support vertical layout (icon view mode: text below icon)
    const QColor& shadowColor() const {
      return shadowColor_;
    }

    void setViewMode(Fm::FolderView::ViewMode mode){
        _viewmode = mode;
    }

    // only support vertical layout (icon view mode: text below icon)
    void setMargins(QSize margins) {
      margins_ = margins.expandedTo(QSize(0, 0));
    }

    QSize getMargins() const {
      return margins_;
    }

    bool hasEditor() const {
        return hasEditor_;
    }

    void setShadowHidden(bool value) {
        shadowHidden_ = value;
    }

    virtual QSize sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const;

    virtual void paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const;

    virtual QWidget* createEditor(QWidget* parent, const QStyleOptionViewItem& option, const QModelIndex& index) const;

    virtual void setEditorData(QWidget* editor, const QModelIndex& index) const;

    virtual bool eventFilter(QObject* object, QEvent* event);

    virtual void updateEditorGeometry(QWidget* editor, const QStyleOptionViewItem& option, const QModelIndex& index) const;

    QSize iconViewTextSize(const QModelIndex& index) const;

private:
    void drawText(QPainter* painter, QStyleOptionViewItem& opt, QRectF& textRect) const;

    static QIcon::Mode iconModeFromState(QStyle::State state);

private:
    QIcon symlinkIcon_;
    QIcon untrustedIcon_;
    QIcon addIcon_;
    QIcon removeIcon_;
    QSize iconSize_;
    QSize itemSize_;
    int fileInfoRole_;
    int iconInfoRole_;
    QColor shadowColor_;
    QSize margins_;
    bool shadowHidden_;
    mutable bool hasEditor_;


    Fm::FolderView::ViewMode _viewmode;

    QString totsize_ = "";
    QString freesize_ = "";
    mutable std::shared_ptr<Folder> folder {nullptr};
    static QHash<QString,FolderModelItemImple*> folderlist_;
};

}

#endif // FM_FOLDERITEMDELEGATE_H
