/*
 * Copyright (C) 2013 - 2015  Hong Jen Yee (PCMan) <pcman.tw@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


#include "mountoperationpassworddialog_p.h"
#include "ui_mount-operation-password.h"
#include "mountoperation.h"
#include <QSettings>
#include <QStandardPaths>
#include <QDir>
#include <QPushButton>

namespace Fm {

MountOperationPasswordDialog::MountOperationPasswordDialog(MountOperation* op, GAskPasswordFlags flags):
    QDialog(),
    mountOperation(op),
    needPassword(flags & G_ASK_PASSWORD_NEED_PASSWORD ? true : false),
    needUserName(flags & G_ASK_PASSWORD_NEED_USERNAME ? true : false),
    needDomain(flags & G_ASK_PASSWORD_NEED_DOMAIN ? true : false),
    canSavePassword(flags & G_ASK_PASSWORD_SAVING_SUPPORTED ? true : false),
    canAnonymous(flags & G_ASK_PASSWORD_ANONYMOUS_SUPPORTED ? true : false) {

    ui = new Ui::MountOperationPasswordDialog();
    ui->setupUi(this);

    ui->buttonBox->button(QDialogButtonBox::Ok)->setText(tr("OK"));
    ui->buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("Cancel"));

    // change the text of Ok button to Connect
    ui->buttonBox->buttons().constFirst()->setText(tr("&Connect"));
    connect(ui->Anonymous, &QAbstractButton::toggled, this, &MountOperationPasswordDialog::onAnonymousToggled);

    if(canAnonymous) {
        // select ananymous by default if applicable.
        ui->Anonymous->setChecked(true);
    }
    else {
        ui->Anonymous->setEnabled(false);
        ui->asUser->setChecked(true);
    }
    if(!needUserName) {
        ui->username->setEnabled(false);
    }
    if(needPassword) {
        if(!needUserName) {
            ui->password->setFocus();
        }
    }
    else {
        ui->password->setEnabled(false);
    }
    if(!needDomain) {
        ui->domain->hide();
        ui->domainLabel->hide();
    }
    scheme = op->uripath().section(':',0,0);
}

MountOperationPasswordDialog::~MountOperationPasswordDialog() {
    delete ui;
}

void MountOperationPasswordDialog::onAnonymousToggled(bool checked) {
    // disable username/password entries if anonymous mode is used
    bool useUserPassword = !checked;
    if(needUserName) {
        ui->username->setEnabled(useUserPassword);
    }
    if(needPassword) {
        ui->password->setEnabled(useUserPassword);
    }
    if(needDomain) {
        ui->domain->setEnabled(useUserPassword);
    }

//    if(canSavePassword) {
//        ui->forgetPassword->setEnabled(useUserPassword);
//        ui->sessionPassword->setEnabled(useUserPassword);
//        ui->storePassword->setEnabled(useUserPassword);
//    }
}

void MountOperationPasswordDialog::setMessage(QString message) {
    domain = message.mid(4,15).simplified();
    ui->message->setText(message);
}

void MountOperationPasswordDialog::setDefaultDomain(QString domain) {
    ui->domain->setText(domain);
}

void MountOperationPasswordDialog::setDefaultUser(QString user) {
    ui->username->setText(user);
}

void MountOperationPasswordDialog::setDefaultPasswd(bool init) {
    QString dirName;
    if(geteuid() != 0) { // non-root user
        dirName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
    }
    if(dirName.isEmpty()) {
        dirName = QDir::homePath() % QLatin1String("/.config");
    }

    QString configFilePath = dirName + "/nde-fileman/nde/connectFavoriteInfos";

    QFile fileName(configFilePath);
    if (!fileName.exists())
    {
        fileName.open(QIODevice::ReadWrite|QIODevice::Text);
        fileName.close();
    }

    QSettings settings(configFilePath, QSettings::IniFormat);
    QStringList connectPwdData_ = settings.value("ConnectPwd").toStringList();

    QList<connectPwdData> listData;
    for (int i = 0; i < connectPwdData_.count(); i++)
    {
        connectPwdData temp;
        temp.host           = connectPwdData_.at(i).section('\n',0,0);
        temp.scheme           = connectPwdData_.at(i).section('\n',1,1);
        temp.userName         = connectPwdData_.at(i).section('\n',2,2);
        temp.passWord           = connectPwdData_.at(i).section('\n',3,3);
        listData.append(temp);
    }

    if (init){
        //初始化界面狀態
        for (int i = 0; i < listData.count(); i++)
        {
            if (domain == listData.at(i).host && ui->username->text() == listData.at(i).userName && scheme == listData.at(i).scheme)
            {
                ui->password->setText(listData.at(i).passWord);
                ui->checkBox_passwd->setChecked(true);
                return ;
            }
        }
    }else {
        QStringList pwdTemp;
        QStringList saveListTemp;

        //判断数据是否已经存在
        //已存在的数据覆盖新密码
        bool dataExists = false;

        for (int i = 0; i < listData.count(); i++)
        {
            if (domain == listData.at(i).host && ui->username->text() == listData.at(i).userName && scheme == listData.at(i).scheme){
                dataExists = true;
                pwdTemp.append(ui->password->text());
            }else {
                pwdTemp.append(listData.at(i).passWord);
            }
        }

        if (dataExists) {
            if (ui->checkBox_passwd->isChecked()){
                for (int i = 0; i < listData.count(); i++) {
                    QString item = listData.at(i).host+"\n"+ scheme+"\n"+listData.at(i).userName+"\n"+ pwdTemp.at(i);
                    saveListTemp << item;
                }
            }else{
                //不勾选保存密码，清空处理
                for (int i = 0; i < listData.count(); i++) {
                    if (domain == listData.at(i).host && ui->username->text() == listData.at(i).userName && scheme == listData.at(i).scheme){
                        continue;
                    }

                    QString item = listData.at(i).host+"\n"+ scheme+"\n"+listData.at(i).userName+"\n"+ pwdTemp.at(i);
                    saveListTemp << item;
                }
            }

        }else {
            //新增数据，保存旧数据
            qCritical() << "new data" ;
            for (int i = 0; i < listData.count(); i++) {
                QString item = listData.at(i).host+"\n"+listData.at(i).scheme+"\n"+listData.at(i).userName+"\n"+ pwdTemp.at(i);
                saveListTemp << item;
            }

            if (ui->checkBox_passwd->isChecked()){
                saveListTemp << domain+"\n"+scheme+"\n"+ui->username->text()+"\n"+ui->password->text();
            }
        }


        connectPwdData_.clear();
        connectPwdData_ = saveListTemp;
        settings.setValue("ConnectPwd", connectPwdData_);
    }

}

void MountOperationPasswordDialog::done(int r) {
    GMountOperation* gmop = mountOperation->mountOperation();


    if(r == QDialog::Accepted) {

        if(needUserName) {
            g_mount_operation_set_username(gmop, ui->username->text().toUtf8());
        }
        if(needDomain) {
            g_mount_operation_set_domain(gmop, ui->domain->text().toUtf8());
        }
        if(needPassword) {
            g_mount_operation_set_password(gmop, ui->password->text().toUtf8());
        }
        if(canAnonymous) {
            g_mount_operation_set_anonymous(gmop, ui->Anonymous->isChecked());
        }

        g_mount_operation_reply(gmop, G_MOUNT_OPERATION_HANDLED);

        //save passwd
        setDefaultPasswd(false);
    }
    else {
        g_mount_operation_reply(gmop, G_MOUNT_OPERATION_ABORTED);
    }
    QDialog::done(r);
}

} // namespace Fm
